'use strict';

let BemMysqlCrud = require('bem-crud-core-mysql');
let BemOrderForm = require('bem-order-form');
let BemStock = require('bem-stock');
let cache = {};
const STATUS = {
  'NON_PAYMENT':0,
  'PAID':1,
  'CANCEL':2
}

/**
 * status: 0为未支付，1为支付成功,2为取消支付
 */

 var transaction = function(connection) {
  return function(done) {
    connection.transaction(function(t) {
        done(null, t);
      })
      .then(function() {
        // it worked
        done();
      })
      .catch(function(e) {
        // it failed
        console.log(e);
        done();
      });
  }
}


class PayMent{
  constructor(config){
    config = config || {};
    this.crud = config.crud;
    this.host = config.host;
    this.db = config.db;
    this.table = config.table;
    this.auth = config.auth;
    this.uid = config.uid;

    // this.amounts = config.amounts || 0;
    // this.payId = config.payId;                 //支付编号
    this.status = STATUS.NON_PAYMENT;             //付款状态
  }

  static * get(config){
    config.name = `${config.app}/${config.table}/${config.uid}`;
    let instance = cache[config.name];
    if (!instance) {
      let crud = yield BemMysqlCrud.get({
        host: config.host,
        db: config.db,
        table: config.table,
        uid: config.uid,
        auth: config.auth
      });
      config.crud = config.crud || crud;
      instance = cache[config.name] = new this(config);
    }
    return instance;
  }

  //获取库存
  * _getStock(){
    let stock = yield BemStock.get({
      host: this.host,
      db: this.db,
      table: 'goods_list',
      auth: this.auth
    });
    return stock;
  }

  * _getOrderForm(){
    let order = yield BemOrderForm.get({
      host: this.host,
      db: this.db,
      table: 'order_form',
      auth: this.auth
    });
    return order;
  }

  //生成支付订单
  * create(options){
    let payId = options.payId;
    let amounts = options.amounts;
    let result = yield this.crud.create({
      docs:{
        payId:payId,
        amounts:amounts
      }
    });
    return true;
  }

  //支付成功
  * submit(options){
    let payId = options.payId;
    let stock = yield this._getStock();
    let order = yield this._getOrderForm();
    //事务
    let tsc = yield transaction(this.crud.conn);
    try {
      let data = yield this.crud.read({
        query:{
          payId:payId,
          status:STATUS.NON_PAYMENT
        }
      });
      if(data.length>0){
        data = data[0];
      }else{
        throw '支付失败';
      }

      let orderlist = yield order.read({
        query:{
          payId:data.payId
        }
      });
      //检查库存
      for(let i=0;i<orderlist.length;i++){
        yield stock.checkUpdate(orderlist[i]);
      }
      //改变商品快照状态
      let result = yield this.crud.update({
        query:{
          payId:payId,
          status:STATUS.NON_PAYMENT
        },
        doc:{
          status:STATUS.PAID
        }
      });

      for(var i=0;i<orderlist.length;i++){
        yield order.update(orderlist[i].orderId,STATUS.PAID);
      }
      yield tsc.commit();
      return true;
    }catch(e){
      yield tsc.rollback(e);
      return false;
    }
  }

  //取消支付订单
  * cancel(options){
    let payId = options.payId;
    let result = yield this.crud.update({
      query:{
        payId:payId,
        status:STATUS.NON_PAYMENT
      },
      doc:{
        status:STATUS.CANCEL
      }
    });
    return true;
  }
}

module.exports = PayMent;
